@@ -1,6 +1,6 @@
 /*  This file is part of the KDE project.
 
-    Copyright (C) 2007 Trolltech ASA. All rights reserved.
+    Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -135,6 +135,15 @@ void X11Renderer::scaleModeChanged(Phonon::VideoWidget::ScaleMode)
     }
 }
 
+void X11Renderer::movieSizeChanged(const QSize &movieSize)
+{
+    Q_UNUSED(movieSize);
+
+    if (m_renderWidget) {
+        m_renderWidget->setGeometry(m_videoWidget->calculateDrawFrameRect());
+    }
+}
+
 bool X11Renderer::eventFilter(QEvent *e)
 {
     if (e->type() == QEvent::Show) {
