@@ -2,18 +2,21 @@
     Copyright (C) 2006-2008 Matthias Kretz <kretz@kde.org>
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), Trolltech ASA 
+    (or its successors, if any) and the KDE Free Qt Foundation, which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    You should have received a copy of the GNU Lesser General Public 
+    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -35,9 +38,7 @@ QT_BEGIN_NAMESPACE
 namespace Phonon
 {
 
-GlobalConfig::GlobalConfig(QObject *parent)
-    : QObject(parent)
-    , m_config(QLatin1String("kde.org"), QLatin1String("libphonon"))
+GlobalConfig::GlobalConfig() : m_config(QLatin1String("kde.org"), QLatin1String("libphonon"))
 {
 }
 
@@ -47,7 +48,8 @@ GlobalConfig::~GlobalConfig()
 
 enum WhatToFilter {
     FilterAdvancedDevices = 1,
-    FilterHardwareDevices = 2
+    FilterHardwareDevices = 2,
+    FilterUnavailableDevices = 4
 };
 
 static void filter(ObjectDescriptionType type, BackendInterface *backendIface, QList<int> *list, int whatToFilter)
@@ -70,6 +72,13 @@ static void filter(ObjectDescriptionType type, BackendInterface *backendIface, Q
                 continue;
             }
         }
+        if (whatToFilter & FilterUnavailableDevices) {
+            var = properties.value("available");
+            if (var.isValid() && !var.toBool()) {
+                it.remove();
+                continue;
+            }
+        }
     }
 }
 
@@ -118,20 +127,18 @@ static QList<int> listSortedByConfig(const QSettingsGroup &backendConfig, Phonon
     return deviceList;
 }
 
-QList<int> GlobalConfig::audioOutputDeviceListFor(Phonon::Category category, HideAdvancedDevicesOverride override) const
+QList<int> GlobalConfig::audioOutputDeviceListFor(Phonon::Category category, int override) const
 {
     //The devices need to be stored independently for every backend
     const QSettingsGroup backendConfig(&m_config, QLatin1String("AudioOutputDevice")); // + Factory::identifier());
     const QSettingsGroup generalGroup(&m_config, QLatin1String("General"));
-    const bool hideAdvancedDevices = (override == FromSettings
+    const bool hideAdvancedDevices = ((override & AdvancedDevicesFromSettings)
             ? generalGroup.value(QLatin1String("HideAdvancedDevices"), true)
-            : static_cast<bool>(override));
-
-    PlatformPlugin *platformPlugin = Factory::platformPlugin();
-    BackendInterface *backendIface = qobject_cast<BackendInterface *>(Factory::backend());
+            : static_cast<bool>(override & HideAdvancedDevices));
 
     QList<int> defaultList;
-    if (platformPlugin) {
+#ifndef QT_NO_PHONON_PLATFORMPLUGIN
+    if (PlatformPlugin *platformPlugin = Factory::platformPlugin()) {
         // the platform plugin lists the audio devices for the platform
         // this list already is in default order (as defined by the platform plugin)
         defaultList = platformPlugin->objectDescriptionIndexes(Phonon::AudioOutputDeviceType);
@@ -146,16 +153,18 @@ QList<int> GlobalConfig::audioOutputDeviceListFor(Phonon::Category category, Hid
             }
         }
     }
+#endif //QT_NO_PHONON_PLATFORMPLUGIN
 
     // lookup the available devices directly from the backend (mostly for virtual devices)
-    if (backendIface) {
+    if (BackendInterface *backendIface = qobject_cast<BackendInterface *>(Factory::backend())) {
         // this list already is in default order (as defined by the backend)
         QList<int> list = backendIface->objectDescriptionIndexes(Phonon::AudioOutputDeviceType);
-        if (hideAdvancedDevices || !defaultList.isEmpty()) {
+        if (hideAdvancedDevices || !defaultList.isEmpty() || (override & HideUnavailableDevices)) {
             filter(AudioOutputDeviceType, backendIface, &list,
                     (hideAdvancedDevices ? FilterAdvancedDevices : 0)
                     // the platform plugin already provided the hardware devices
                     | (defaultList.isEmpty() ? 0 : FilterHardwareDevices)
+                    | ((override & HideUnavailableDevices) ? FilterUnavailableDevices : 0)
                     );
         }
         defaultList += list;
@@ -164,28 +173,27 @@ QList<int> GlobalConfig::audioOutputDeviceListFor(Phonon::Category category, Hid
     return listSortedByConfig(backendConfig, category, defaultList);
 }
 
-int GlobalConfig::audioOutputDeviceFor(Phonon::Category category) const
+int GlobalConfig::audioOutputDeviceFor(Phonon::Category category, int override) const
 {
-    QList<int> ret = audioOutputDeviceListFor(category);
+    QList<int> ret = audioOutputDeviceListFor(category, override);
     if (ret.isEmpty())
         return -1;
     return ret.first();
 }
 
-QList<int> GlobalConfig::audioCaptureDeviceListFor(Phonon::Category category, HideAdvancedDevicesOverride override) const
+#ifndef QT_NO_PHONON_AUDIOCAPTURE
+QList<int> GlobalConfig::audioCaptureDeviceListFor(Phonon::Category category, int override) const
 {
     //The devices need to be stored independently for every backend
     const QSettingsGroup backendConfig(&m_config, QLatin1String("AudioCaptureDevice")); // + Factory::identifier());
     const QSettingsGroup generalGroup(&m_config, QLatin1String("General"));
-    const bool hideAdvancedDevices = (override == FromSettings
+    const bool hideAdvancedDevices = ((override & AdvancedDevicesFromSettings)
             ? generalGroup.value(QLatin1String("HideAdvancedDevices"), true)
-            : static_cast<bool>(override));
-
-    PlatformPlugin *platformPlugin = Factory::platformPlugin();
-    BackendInterface *backendIface = qobject_cast<BackendInterface *>(Factory::backend());
+            : static_cast<bool>(override & HideAdvancedDevices));
 
     QList<int> defaultList;
-    if (platformPlugin) {
+#ifndef QT_NO_PHONON_PLATFORMPLUGIN
+    if (PlatformPlugin *platformPlugin = Factory::platformPlugin()) {
         // the platform plugin lists the audio devices for the platform
         // this list already is in default order (as defined by the platform plugin)
         defaultList = platformPlugin->objectDescriptionIndexes(Phonon::AudioCaptureDeviceType);
@@ -200,16 +208,18 @@ QList<int> GlobalConfig::audioCaptureDeviceListFor(Phonon::Category category, Hi
             }
         }
     }
+#endif //QT_NO_PHONON_PLATFORMPLUGIN
 
     // lookup the available devices directly from the backend (mostly for virtual devices)
-    if (backendIface) {
+    if (BackendInterface *backendIface = qobject_cast<BackendInterface *>(Factory::backend())) {
         // this list already is in default order (as defined by the backend)
         QList<int> list = backendIface->objectDescriptionIndexes(Phonon::AudioCaptureDeviceType);
-        if (hideAdvancedDevices || !defaultList.isEmpty()) {
+        if (hideAdvancedDevices || !defaultList.isEmpty() || (override & HideUnavailableDevices)) {
             filter(AudioCaptureDeviceType, backendIface, &list,
                     (hideAdvancedDevices ? FilterAdvancedDevices : 0)
                     // the platform plugin already provided the hardware devices
                     | (defaultList.isEmpty() ? 0 : FilterHardwareDevices)
+                    | ((override & HideUnavailableDevices) ? FilterUnavailableDevices : 0)
                   );
         }
         defaultList += list;
@@ -218,18 +228,16 @@ QList<int> GlobalConfig::audioCaptureDeviceListFor(Phonon::Category category, Hi
     return listSortedByConfig(backendConfig, category, defaultList);
 }
 
-int GlobalConfig::audioCaptureDeviceFor(Phonon::Category category) const
+int GlobalConfig::audioCaptureDeviceFor(Phonon::Category category, int override) const
 {
-    QList<int> ret = audioCaptureDeviceListFor(category);
+    QList<int> ret = audioCaptureDeviceListFor(category, override);
     if (ret.isEmpty())
         return -1;
     return ret.first();
 }
+#endif //QT_NO_PHONON_AUDIOCAPTURE
+
 
 } // namespace Phonon
 
 QT_END_NAMESPACE
-
-#include "moc_globalconfig_p.cpp"
-
-// vim: sw=4 ts=4
