@@ -4,7 +4,11 @@
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of
-    the License, or (at your option) version 3.
+    the License or (at your option) version 3 or any later version
+    accepted by the membership of KDE e.V. (or its successor approved
+    by the membership of KDE e.V.), Trolltech ASA (or its successors, 
+    if any) and the KDE Free Qt Foundation, which shall act as a proxy 
+    defined in Section 14 of version 3 of the license.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
